package service

import (
	"electricity_bill_calc/global"
	"electricity_bill_calc/model"
	"electricity_bill_calc/utils"
	"time"

	"github.com/google/uuid"
	"xorm.io/builder"
)

type _ReportService struct{}

var ReportService _ReportService

func (_ReportService) FetchParksWithNewestReport(uid string) ([]model.ParkNewestReport, error) {
	parks := make([]model.ParkNewestReport, 0)
	err := global.DBConn.
		Alias("p").
		Join("LEFT", []string{"report", "r"}, "r.park_id=p.id").
		Where(builder.Eq{"p.user_id": uid, "p.enabled": true}).
		Find(&parks)
	if err != nil {
		return make([]model.ParkNewestReport, 0), err
	}
	reducedParks := utils.Reduce(
		parks,
		make(map[string]model.ParkNewestReport, 0),
		func(acc map[string]model.ParkNewestReport, elem model.ParkNewestReport) map[string]model.ParkNewestReport {
			if v, ok := acc[elem.Park.Id]; ok {
				if elem.Report != nil {
					if v.Report == nil || (elem.Report.Period.After(v.Report.Period)) {
						acc[elem.Park.Id] = elem
					}
				}
			} else {
				acc[elem.Park.Id] = elem
			}
			return acc
		},
	)
	return utils.Values(reducedParks), nil
}

func (_ReportService) IsNewPeriodValid(uid string, period time.Time) (bool, error) {
	reports := make([]model.Report, 0)
	err := global.DBConn.
		Table("report").Alias("r").
		Join("INNER", []string{"park", "p"}, "r.park_id=p.id").
		Where(builder.Eq{"p.user_id": uid}).
		Find(&reports)
	if err != nil {
		return false, nil
	}
	// 检查给定的期数在目前的记录中是否已经存在
	exists := utils.Reduce(
		reports,
		false,
		func(acc bool, elem model.Report) bool {
			if elem.Period.Equal(period) {
				return acc || true
			} else {
				return acc || false
			}
		},
	)
	if exists {
		return false, nil
	}
	// 检查给定的期数与目前已发布的最大期数的关系
	maxPublished := utils.Reduce(
		reports,
		nil,
		func(acc *time.Time, elem model.Report) *time.Time {
			if elem.Published {
				if acc == nil || (acc != nil && elem.Period.After(*acc)) {
					return &elem.Period
				}
			}
			return acc
		},
	)
	// 检查给定的期数与目前未发布的最大期数的关系
	maxUnpublished := utils.Reduce(
		reports,
		nil,
		func(acc *time.Time, elem model.Report) *time.Time {
			if acc == nil || (acc != nil && elem.Period.After(*acc)) {
				return &elem.Period
			}
			return acc
		},
	)
	if maxUnpublished == nil {
		return true, nil
	}
	if maxPublished != nil && maxUnpublished.Equal(*maxPublished) {
		// 此时不存在未发布的报表
		return utils.IsNextMonth(*maxPublished, period), nil
	} else {
		// 存在未发布的报表
		return false, nil
	}
}

func (_ReportService) InitializeNewReport(report model.Report) (err error) {
	report.Id = uuid.New().String()
	report.StepState = model.NewSteps()
	report.Published = false
	report.Withdraw = model.REPORT_NOT_WITHDRAW
	_, err = global.DBConn.Insert(report)
	if err != nil {
		return
	}
	return nil
}

func (_ReportService) RetreiveReportIndex(rid string) (*model.Report, error) {
	reports := make([]model.Report, 0)
	err := global.DBConn.Where(builder.Eq{"id": rid}).Find(&reports)
	if len(reports) > 0 {
		return &reports[0], nil
	} else {
		return nil, err
	}
}
